package com.javainuse.bank.service;

import java.io.FileInputStream;
import java.io.IOException;

import org.springframework.stereotype.Service;

import com.javainuse.banking.AddressProofRequest;
import com.javainuse.banking.BankServiceGrpc;

import io.grpc.ManagedChannel;
import io.grpc.ManagedChannelBuilder;
import io.grpc.stub.StreamObserver;

@Service
public class BankService {

	public void uploadAddressProof() {

		String filePath = "C:\\test\\test.pdf";
		String accountHolderName = "javainuse";
		String accountNumber = "account5";
		// Create a channel to connect to the gRPC server
		ManagedChannel channel = ManagedChannelBuilder.forAddress("localhost", 8090).usePlaintext().build();

		// Create a gRPC stub
		BankServiceGrpc.BankServiceStub stub = BankServiceGrpc.newStub(channel);

		// Create an instance of the ResponseObserver class to handle the response from
		// the server
		ResponseObserver responseObserver = new ResponseObserver();

		// Create a StreamObserver to send the request to the server
		StreamObserver<AddressProofRequest> requestObserver = stub.uploadAddressProof(responseObserver);

		// Read the PDF file from disk and send it to the server in chunks
		try (FileInputStream fileInputStream = new FileInputStream(filePath)) {
			byte[] buffer = new byte[1024];
			int bytesRead;
			while ((bytesRead = fileInputStream.read(buffer)) != -1) {
				// Create an AddressProofRequest with the account holder name, account number,
				// and file chunk
				AddressProofRequest request = AddressProofRequest.newBuilder().setAccountHolderName(accountHolderName)
						.setAccountNumber(accountNumber)
						.setPdfFile(com.google.protobuf.ByteString.copyFrom(buffer, 0, bytesRead)).build();

				// Send the request to the server
				requestObserver.onNext(request);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		// Mark the client-side stream as complete
		requestObserver.onCompleted();

		// Wait for the response from the server
		responseObserver.awaitCompletion();

		// Close the channel after the upload is completed
		channel.shutdown();
	}
}
